<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/03/19
 * Time: 14.49
 */

namespace FiloBlu\RuleDayFromBirthday\Model\Rule\Condition\Customer;




use Magento\SalesRule\Api\CouponRepositoryInterface;
use Magento\SalesRule\Model\Coupon;
use Magento\Framework\App\RequestInterface;
use Magento\Checkout\Model\Session;
use Magento\Framework\Message\ManagerInterface;

class Birthday extends \Magento\Rule\Model\Condition\AbstractCondition
{


    protected $couponRepository;
    protected $coupon;
    protected $request;
    protected $session;
    protected $manager;

    /**
     * Birthday constructor.
     * @param \Magento\Rule\Model\Condition\Context $context
     * @param CouponRepositoryInterface $couponRepository
     * @param Coupon $coupon
     * @param RequestInterface $request
     * @param Session $session
     * @param ManagerInterface $manager
     * @param array $data
     */
    public function __construct(
        \Magento\Rule\Model\Condition\Context $context,
        CouponRepositoryInterface $couponRepository,
        Coupon $coupon,
        RequestInterface $request,
        Session $session,
        ManagerInterface $manager,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->couponRepository = $couponRepository;
        $this->coupon = $coupon;
        $this->request = $request;
        $this->session = $session;
        $this->manager = $manager;
    }
    /**
     * Load attribute options
     *
     * @return $this
     */
    public function loadAttributeOptions()
    {
        $attributes = [
            'customer_birthday' => __("Customer Birthday")
        ];
        $this->setAttributeOption($attributes);
        return $this;
    }
    /**
     * Get input type
     * Possible values are: string, numeric, date, select, multiselect, grid, boolean
     *
     * @return string|void
     */
    public function getInputType()
    {
        return 'boolean';
    }
    /**
     * Get value element type
     *
     * We have the value element types as select, text
     * @return string
     */
    public function getValueElementType()
    {
        return 'select';
    }

    /**
     * Get value select options
     *
     * @return array|mixed
     */
    public function getValueSelectOptions()
    {
        if (!$this->hasData('value_select_options')) {
            $options = ["True" => "True"];

            $this->setData('value_select_options', $options);
        }
        return $this->getData('value_select_options');
    }

    /**
     * @param int $customerId
     * @return string|null
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCustomerDobById(int $customerId)
    {
        return $this->customerRepository->getById($customerId)->getDob();
    }


    /**
     * @param $dateString
     * @return string
     */
    protected function updateDate($dateString)
    {
        $date_arr = date_parse($dateString);
        $date_arr['year'] = date('Y');

        return $date_arr['year']."-".$date_arr['month']."-".$date_arr['day'];
    }


    /**
     * @param $dob
     * @return int
     */
    protected function getDaysFromBirthday($dob)
    {
        $new_dob = $this->updateDate($dob);
        $interval = strtotime("now") - strtotime($new_dob);
        $num_of_days = (int) ($interval/86400);
        return $num_of_days;
    }

    /**
     * @param AbstractModel $model
     * @return bool
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */

    public function validate(\Magento\Framework\Model\AbstractModel $model)
    {


        $coupon_code = $this->session->getQuote()->getCouponCode();
        if(!$coupon_code)
        {
            $val = "False";
            $model->setData('customer_birthday', $val);
            return parent::validate($model);
        }

        try
        {
            $coupon_id = $this->coupon->loadByCode($coupon_code)->getCouponId();
            $validFrom = $this->couponRepository->getById($coupon_id)->getCreatedAt();
            if(strtotime($validFrom) <= strtotime(date('Y-m-d')))
            {
                $val = "True";
            }
            else
            {
                $val = "False";
            }


            $model->setData('customer_birthday', $val);
            return parent::validate($model);
        }
        catch (\Exception $e)
        {
            $this->manager->addErrorMessage($e);
        }


    }
}